@@ -1,4 +1,4 @@
-/*
+/* -*- c -*-
  * libvirt.h:
  * Summary: core interfaces for the libvirt library
  * Description: Provides the interfaces of the libvirt library to handle
@@ -113,30 +113,6 @@ struct _virDomainInfo {
 
 typedef virDomainInfo *virDomainInfoPtr;
 
-/**
- * virDomainKernel:
- *
- * a virDomainImage is the set of kernel related informations associated
- * to a domain
- */
-
-typedef struct _virDomainKernel virDomainKernel;
-
-struct _virDomainKernel {
-    const char *kernel;		/* filename pointing to the kernel image */
-    const char *ramdisk;	/* an optional init ramdisk */
-    const char *root;		/* an optional root block device */
-    const char *extra;		/* optional kernel command line parameters */
-};
-
-/**
- * virDomainKernelPtr:
- *
- * a virDomainKernelPtr is a pointer to a virDomainKernel structure.
- */
-
-typedef virDomainKernel *virDomainKernelPtr;
-
 /**
  * virDomainCreateFlags:
  *
@@ -168,6 +144,71 @@ struct _virNodeInfo {
 };
 
 
+/**
+ * virDomainSchedParameterType:
+ *
+ * A scheduler parameter field type
+ */
+typedef enum {
+    VIR_DOMAIN_SCHED_FIELD_INT     = 1,	/* integer case */
+    VIR_DOMAIN_SCHED_FIELD_UINT    = 2,	/* unsigned integer case */
+    VIR_DOMAIN_SCHED_FIELD_LLONG   = 3,	/* long long case */
+    VIR_DOMAIN_SCHED_FIELD_ULLONG  = 4,	/* unsigned long long case */
+    VIR_DOMAIN_SCHED_FIELD_DOUBLE  = 5,	/* double case */
+    VIR_DOMAIN_SCHED_FIELD_BOOLEAN = 6	/* boolean(character) case */
+} virSchedParameterType;
+
+/**
+ * VIR_DOMAIN_SCHED_FIELD_LENGTH:
+ *
+ * Macro providing the field length of virSchedParameter
+ */
+
+#define VIR_DOMAIN_SCHED_FIELD_LENGTH 80
+
+/**
+ * virDomainSchedParameter:
+ *
+ * a virDomainSchedParameter is the set of scheduler parameters
+ */
+
+typedef struct _virSchedParameter virSchedParameter;
+
+struct _virSchedParameter {
+    char field[VIR_DOMAIN_SCHED_FIELD_LENGTH];	/* parameter name */
+    int type;	/* parameter type */
+    union {
+        int i;				/* data for integer case */
+        unsigned int ui;	/* data for unsigned integer case */
+        long long int l;	/* data for long long integer case */
+        unsigned long long int ul;	/* data for unsigned long long integer case */
+        double d;	/* data for double case */
+        char b;		/* data for char case */
+    } value; /* parameter value */
+};
+
+/**
+ * virSchedParameterPtr:
+ *
+ * a virSchedParameterPtr is a pointer to a virSchedParameter structure.
+ */
+
+typedef virSchedParameter *virSchedParameterPtr;
+
+/*
+ * Fetch scheduler parameters, caller allocates 'params' field of size 'nparams'
+ */
+int	virDomainGetSchedulerParameters	(virDomainPtr domain,
+					 virSchedParameterPtr params,
+					 int *nparams);
+
+/*
+ * Change scheduler parameters
+ */
+int	virDomainSetSchedulerParameters	(virDomainPtr domain,
+					 virSchedParameterPtr params,
+					 int nparams);
+
 /**
  * VIR_NODEINFO_MAXCPUS:
  * @nodeinfo: virNodeInfo instance
@@ -187,6 +228,24 @@ struct _virNodeInfo {
 
 typedef virNodeInfo *virNodeInfoPtr;
 
+/**
+ * VIR_UUID_BUFLEN:
+ *
+ * This macro provides the length of the buffer required
+ * for virDomainGetUUID()
+ */
+
+#define VIR_UUID_BUFLEN (16)
+
+/**
+ * VIR_UUID_STRING_BUFLEN:
+ *
+ * This macro provides the length of the buffer required
+ * for virDomainGetUUIDString()
+ */
+
+#define VIR_UUID_STRING_BUFLEN (36+1)
+
 /* library versionning */
 
 /**
@@ -196,7 +255,7 @@ typedef virNodeInfo *virNodeInfoPtr;
  * version * 1,000,000 + minor * 1000 + micro
  */
 
-#define LIBVIR_VERSION_NUMBER 1008
+#define LIBVIR_VERSION_NUMBER 3000
 
 int			virGetVersion		(unsigned long *libVer,
 						 const char *type,
@@ -213,8 +272,19 @@ int			virConnectClose		(virConnectPtr conn);
 const char *		virConnectGetType	(virConnectPtr conn);
 int			virConnectGetVersion	(virConnectPtr conn,
 						 unsigned long *hvVer);
+char *                  virConnectGetHostname   (virConnectPtr conn);
+char *                  virConnectGetURI        (virConnectPtr conn);
+
+
+/*
+ * Capabilities of the connection / driver.
+ */
+
+int                     virConnectGetMaxVcpus   (virConnectPtr conn,
+						 const char *type);
 int			virNodeGetInfo		(virConnectPtr conn,
 						 virNodeInfoPtr info);
+char *                  virConnectGetCapabilities (virConnectPtr conn);
 
 /*
  * Gather list of running domains
@@ -229,6 +299,11 @@ int			virConnectListDomains	(virConnectPtr conn,
 int			virConnectNumOfDomains	(virConnectPtr conn);
 
 
+/*
+ * Get connection from domain.
+ */
+virConnectPtr		virDomainGetConnect     (virDomainPtr domain);
+
 /*
  * Domain creation and destruction
  */
@@ -264,12 +339,25 @@ int			virDomainSave		(virDomainPtr domain,
 int			virDomainRestore	(virConnectPtr conn,
 						 const char *from);
 
+/*
+ * Domain core dump
+ */
+int			virDomainCoreDump	(virDomainPtr domain,
+						 const char *to,
+						 int flags);
+
 /*
  * Domain runtime informations
  */
 int			virDomainGetInfo	(virDomainPtr domain,
 						 virDomainInfoPtr info);
 						 
+/*
+ * Return scheduler type in effect 'sedf', 'credit', 'linux'
+ */
+char * 			virDomainGetSchedulerType(virDomainPtr domain,
+						 int *nparams);
+
 /*
  * Dynamic control of domains
  */
@@ -285,6 +373,8 @@ int			virDomainSetMaxMemory	(virDomainPtr domain,
 						 unsigned long memory);
 int			virDomainSetMemory	(virDomainPtr domain,
 						 unsigned long memory);
+int			virDomainGetMaxVcpus	(virDomainPtr domain);
+
 /*
  * XML domain description
  */
@@ -299,10 +389,15 @@ virDomainPtr		virDomainDefineXML	(virConnectPtr conn,
 int			virDomainUndefine	(virDomainPtr domain);
 int                     virConnectNumOfDefinedDomains  (virConnectPtr conn);
 int			virConnectListDefinedDomains (virConnectPtr conn,
-						 const char **names,
+						 char **const names,
 						 int maxnames);
 int			virDomainCreate		(virDomainPtr domain);
 
+int			virDomainGetAutostart	(virDomainPtr domain,
+						 int *autostart);
+int			virDomainSetAutostart	(virDomainPtr domain,
+						 int autostart);
+
 /**
  * virVcpuInfo: structure for information about a virtual CPU in a domain.
  */
@@ -415,6 +510,103 @@ int			virDomainGetVcpus	(virDomainPtr domain,
  */
 #define VIR_GET_CPUMAP(cpumaps,maplen,vcpu)	&(cpumaps[(vcpu)*(maplen)])
 
+int virDomainAttachDevice(virDomainPtr domain, char *xml);
+int virDomainDetachDevice(virDomainPtr domain, char *xml);
+
+/*
+ * Virtual Networks API
+ */
+
+/**
+ * virNetwork:
+ *
+ * a virNetwork is a private structure representing a virtual network.
+ */
+typedef struct _virNetwork virNetwork;
+
+/**
+ * virNetworkPtr:
+ *
+ * a virNetworkPtr is pointer to a virNetwork private structure, this is the
+ * type used to reference a virtual network in the API.
+ */
+typedef virNetwork *virNetworkPtr;
+
+/*
+ * Get connection from network.
+ */
+virConnectPtr		virNetworkGetConnect    (virNetworkPtr network);
+
+/*
+ * List active networks
+ */
+int			virConnectNumOfNetworks	(virConnectPtr conn);
+int			virConnectListNetworks	(virConnectPtr conn,
+						 char **const names,
+						 int maxnames);
+
+/*
+ * List inactive networks
+ */
+int			virConnectNumOfDefinedNetworks	(virConnectPtr conn);
+int			virConnectListDefinedNetworks	(virConnectPtr conn,
+							 char **const names,
+							 int maxnames);
+
+/*
+ * Lookup network by name or uuid
+ */
+virNetworkPtr		virNetworkLookupByName		(virConnectPtr conn,
+							 const char *name);
+virNetworkPtr 		virNetworkLookupByUUID		(virConnectPtr conn,
+							 const unsigned char *uuid);
+virNetworkPtr		virNetworkLookupByUUIDString	(virConnectPtr conn,
+							 const char *uuid);
+
+/*
+ * Create active transient network
+ */
+virNetworkPtr		virNetworkCreateXML	(virConnectPtr conn,
+						 const char *xmlDesc);
+
+/*
+ * Define inactive persistent network
+ */
+virNetworkPtr		virNetworkDefineXML	(virConnectPtr conn,
+						 const char *xmlDesc);
+
+/*
+ * Delete persistent network
+ */
+int			virNetworkUndefine	(virNetworkPtr network);
+
+/*
+ * Activate persistent network
+ */
+int			virNetworkCreate	(virNetworkPtr network);
+
+/*
+ * Network destroy/free
+ */
+int			virNetworkDestroy	(virNetworkPtr network);
+int			virNetworkFree		(virNetworkPtr network);
+
+/*
+ * Network informations
+ */
+const char*		virNetworkGetName	(virNetworkPtr network);
+int			virNetworkGetUUID	(virNetworkPtr network,
+						 unsigned char *uuid);
+int			virNetworkGetUUIDString	(virNetworkPtr network,
+						 char *buf);
+char *			virNetworkGetXMLDesc	(virNetworkPtr network,
+						 int flags);
+char *			virNetworkGetBridgeName (virNetworkPtr network);
+
+int			virNetworkGetAutostart	(virNetworkPtr network,
+						 int *autostart);
+int			virNetworkSetAutostart	(virNetworkPtr network,
+						 int autostart);
 
 #ifdef __cplusplus
 }
