@@ -33,6 +33,18 @@ libvirt_longWrap(long val)
     return (ret);
 }
 
+PyObject *
+libvirt_ulongWrap(unsigned long val)
+{
+    PyObject *ret;
+
+#ifdef DEBUG
+    printf("libvirt_ulongWrap: val = %lu\n", val);
+#endif
+    ret = PyLong_FromLong(val);
+    return (ret);
+}   
+
 PyObject *
 libvirt_longlongWrap(long long val)
 {
@@ -50,13 +62,13 @@ libvirt_charPtrWrap(char *str)
 {
     PyObject *ret;
 
-#ifdef DEBUG
-    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
-#endif
     if (str == NULL) {
         Py_INCREF(Py_None);
         return (Py_None);
     }
+#ifdef DEBUG
+    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
+#endif
     ret = PyString_FromString(str);
     free(str);
     return (ret);
@@ -67,13 +79,13 @@ libvirt_constcharPtrWrap(const char *str)
 {
     PyObject *ret;
 
-#ifdef DEBUG
-    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
-#endif
     if (str == NULL) {
         Py_INCREF(Py_None);
         return (Py_None);
     }
+#ifdef DEBUG
+    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
+#endif
     ret = PyString_FromString(str);
     return (ret);
 }
@@ -83,13 +95,13 @@ libvirt_charPtrConstWrap(const char *str)
 {
     PyObject *ret;
 
-#ifdef DEBUG
-    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
-#endif
     if (str == NULL) {
         Py_INCREF(Py_None);
         return (Py_None);
     }
+#ifdef DEBUG
+    printf("libvirt_xmlcharPtrWrap: str = %s\n", str);
+#endif
     ret = PyString_FromString(str);
     return (ret);
 }
@@ -112,6 +124,24 @@ libvirt_virDomainPtrWrap(virDomainPtr node)
     return (ret);
 }
 
+PyObject *
+libvirt_virNetworkPtrWrap(virNetworkPtr node)
+{
+    PyObject *ret;
+
+#ifdef DEBUG
+    printf("libvirt_virNetworkPtrWrap: node = %p\n", node);
+#endif
+    if (node == NULL) {
+        Py_INCREF(Py_None);
+        return (Py_None);
+    }
+    ret =
+        PyCObject_FromVoidPtrAndDesc((void *) node, (char *) "virNetworkPtr",
+                                     NULL);
+    return (ret);
+}
+
 PyObject *
 libvirt_virConnectPtrWrap(virConnectPtr node)
 {
