@@ -17,7 +17,7 @@
 #include "internal.h"
 
 static virError lastErr =       /* the last error */
-{ 0, 0, NULL, VIR_ERR_NONE, NULL, NULL, NULL, NULL, NULL, 0, 0 };
+{ 0, 0, NULL, VIR_ERR_NONE, NULL, NULL, NULL, NULL, NULL, 0, 0, NULL };
 static virErrorFunc virErrorHandler = NULL;     /* global error handlet */
 static void *virUserData = NULL;        /* associated data */
 
@@ -230,7 +230,7 @@ virConnSetErrorFunc(virConnectPtr conn, void *userData,
 void
 virDefaultErrorFunc(virErrorPtr err)
 {
-    const char *lvl = "", *dom = "", *domain = "";
+    const char *lvl = "", *dom = "", *domain = "", *network = "";
     int len;
 
     if ((err == NULL) || (err->code == VIR_ERR_OK))
@@ -253,6 +253,9 @@ virDefaultErrorFunc(virErrorPtr err)
         case VIR_FROM_XEN:
             dom = "Xen ";
             break;
+        case VIR_FROM_XML:
+            dom = "XML ";
+            break;
         case VIR_FROM_XEND:
             dom = "Xen Daemon ";
             break;
@@ -265,23 +268,42 @@ virDefaultErrorFunc(virErrorPtr err)
         case VIR_FROM_RPC:
             dom = "XML-RPC ";
             break;
+        case VIR_FROM_QEMU:
+            dom = "QEMU ";
+            break;
+        case VIR_FROM_NET:
+            dom = "Network ";
+            break;
+        case VIR_FROM_TEST:
+            dom = "Test ";
+            break;
+        case VIR_FROM_REMOTE:
+            dom = "Remote ";
+            break;
     }
     if ((err->dom != NULL) && (err->code != VIR_ERR_INVALID_DOMAIN)) {
         domain = err->dom->name;
+    } else if ((err->net != NULL) && (err->code != VIR_ERR_INVALID_NETWORK)) {
+        network = err->net->name;
     }
     len = strlen(err->message);
-    if ((len == 0) || (err->message[len - 1] != '\n'))
-        fprintf(stderr, "libvir: %s%s %s: %s\n",
-                dom, lvl, domain, err->message);
+    if ((err->domain == VIR_FROM_XML) && (err->code == VIR_ERR_XML_DETAIL) &&
+        (err->int1 != 0))
+        fprintf(stderr, "libvir: %s%s %s%s: line %d: %s",
+                dom, lvl, domain, network, err->int1, err->message);
+    else if ((len == 0) || (err->message[len - 1] != '\n'))
+        fprintf(stderr, "libvir: %s%s %s%s: %s\n",
+                dom, lvl, domain, network, err->message);
     else
-        fprintf(stderr, "libvir: %s%s %s: %s",
-                dom, lvl, domain, err->message);
+        fprintf(stderr, "libvir: %s%s %s%s: %s",
+                dom, lvl, domain, network, err->message);
 }
 
 /**
  * __virRaiseError:
  * @conn: the connection to the hypervisor if available
  * @dom: the domain if available
+ * @net: the network if available
  * @domain: the virErrorDomain indicating where it's coming from
  * @code: the virErrorNumber code for the error
  * @level: the virErrorLevel for the error
@@ -297,7 +319,7 @@ virDefaultErrorFunc(virErrorPtr err)
  * immediately if a callback is found and store it for later handling.
  */
 void
-__virRaiseError(virConnectPtr conn, virDomainPtr dom,
+__virRaiseError(virConnectPtr conn, virDomainPtr dom, virNetworkPtr net,
                 int domain, int code, virErrorLevel level,
                 const char *str1, const char *str2, const char *str3,
                 int int1, int int2, const char *msg, ...)
@@ -336,6 +358,7 @@ __virRaiseError(virConnectPtr conn, virDomainPtr dom,
     virResetError(to);
     to->conn = conn;
     to->dom = dom;
+    to->net = net;
     to->domain = domain;
     to->code = code;
     to->message = str;
@@ -387,10 +410,10 @@ __virErrorMsg(virErrorNumber error, const char *info)
             errmsg = _("out of memory");
             break;
         case VIR_ERR_NO_SUPPORT:
-            if (info != NULL)
-		errmsg = _("no support for hypervisor");
+            if (info == NULL)
+		errmsg = _("this function is not supported by the hypervisor");
 	    else
-		errmsg = _("no support for hypervisor %s");
+		errmsg = _("this function is not supported by the hypervisor: %s");
             break;
         case VIR_ERR_NO_CONNECT:
             if (info == NULL)
@@ -515,7 +538,7 @@ __virErrorMsg(virErrorNumber error, const char *info)
             else
                 errmsg = _("too many drivers registered in %s");
             break;
-        case VIR_ERR_CALL_FAILED:
+        case VIR_ERR_CALL_FAILED: /* DEPRECATED, use VIR_ERR_NO_SUPPORT */
             if (info == NULL)
                 errmsg = _("library call failed, possibly not supported");
             else
@@ -569,6 +592,74 @@ __virErrorMsg(virErrorNumber error, const char *info)
 	    else
 	        errmsg = _("failed to write configuration file: %s");
             break;
+	case VIR_ERR_XML_DETAIL:
+	    if (info == NULL)
+	        errmsg = _("parser error");
+	    else
+	        errmsg = "%s";
+            break;
+        case VIR_ERR_INVALID_NETWORK:
+            if (info == NULL)
+		errmsg = _("invalid network pointer in");
+	    else
+	        errmsg = _("invalid network pointer in %s");
+            break;
+	case VIR_ERR_NETWORK_EXIST:
+	    if (info == NULL)
+	        errmsg = _("this network exists already");
+	    else
+	        errmsg = _("network %s exists already");
+            break;
+	case VIR_ERR_SYSTEM_ERROR:
+	    if (info == NULL)
+		errmsg = _("system call error");
+	    else
+		errmsg = "%s";
+	    break;
+	case VIR_ERR_RPC:
+	    if (info == NULL)
+		errmsg = _("RPC error");
+	    else
+		errmsg = "%s";
+	    break;
+	case VIR_ERR_GNUTLS_ERROR:
+	    if (info == NULL)
+		errmsg = _("GNUTLS call error");
+	    else
+		errmsg = "%s";
+	    break;
+	case VIR_WAR_NO_NETWORK:
+	    if (info == NULL)
+		errmsg = _("Failed to find the network");
+	    else
+		errmsg = _("Failed to find the network: %s");
+	    break;
+	case VIR_ERR_NO_DOMAIN:
+	    if (info == NULL)
+		errmsg = _("Domain not found");
+	    else
+		errmsg = _("Domain not found: %s");
+	    break;
+	case VIR_ERR_NO_NETWORK:
+	    if (info == NULL)
+		errmsg = _("Network not found");
+	    else
+		errmsg = _("Network not found: %s");
+	    break;
     }
     return (errmsg);
 }
+
+/*
+ * vim: set tabstop=4:
+ * vim: set shiftwidth=4:
+ * vim: set expandtab:
+ */
+/*
+ * Local variables:
+ *  indent-tabs-mode: nil
+ *  c-indent-level: 4
+ *  c-basic-offset: 4
+ *  tab-width: 4
+ * End:
+ */
