@@ -15,7 +15,9 @@
 extern "C" {
 #endif
 
-void		xenStoreRegister	(void);
+extern struct xenUnifiedDriver xenStoreDriver;
+int xenStoreInit (void);
+
 int		xenStoreOpen		(virConnectPtr conn,
 					 const char *name,
 					 int flags);
@@ -26,7 +28,7 @@ int		xenStoreNumOfDomains	(virConnectPtr conn);
 int     	xenStoreListDomains	(virConnectPtr conn,
 					 int *ids,
 					 int maxids);
-virDomainPtr 	xenStoreDomainLookupByName(virConnectPtr conn,
+virDomainPtr 	xenStoreLookupByName(virConnectPtr conn,
 					 const char *name);
 unsigned long 	xenStoreGetMaxMemory	(virDomainPtr domain);
 int		xenStoreDomainSetMemory	(virDomainPtr domain,
@@ -36,8 +38,16 @@ int 		xenStoreDomainShutdown	(virDomainPtr domain);
 int 		xenStoreDomainReboot	(virDomainPtr domain,
 					 unsigned int flags);
 
-int             xenStoreDomainGetVNCPort(virConnectPtr conn, int domid);
-char *          xenStoreDomainGetConsolePath(virConnectPtr conn, int domid);
+/* those are entry point for the proxy */
+int             xenStoreDomainGetVNCPort(virConnectPtr conn,
+					 int domid);
+char *          xenStoreDomainGetConsolePath(virConnectPtr conn,
+					 int domid);
+char *		xenStoreDomainGetOSTypeID(virConnectPtr conn,
+					 int id);
+char *		xenStoreDomainGetNetworkID(virConnectPtr conn,
+					 int id,
+					 const char *mac);
 
 #ifdef __cplusplus
 }
