@@ -1,13 +1,26 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <libvirt/libvirt.h>
+#include <libvirt/virterror.h>
+#include "internal.h"
+
+static void errorHandler(void *userData ATTRIBUTE_UNUSED,
+			 virErrorPtr error ATTRIBUTE_UNUSED) {
+}
 
 int main(void) {
     int id = 0;
+    int ro = 0;
     virConnectPtr conn;
     virDomainPtr dom;
 
-    conn = virConnectOpen("");
+    virSetErrorFunc(NULL, errorHandler);
+
+    conn = virConnectOpen(NULL);
+    if (conn == NULL) {
+        ro = 1;
+	conn = virConnectOpenReadOnly(NULL);
+    }
     if (conn == NULL) {
         fprintf(stderr, "First virConnectOpen() failed\n");
 	exit(1);
@@ -19,7 +32,10 @@ int main(void) {
     }
     virDomainFree(dom);
     virConnectClose(conn);
-    conn = virConnectOpen("");
+    if (ro == 1)
+	conn = virConnectOpenReadOnly(NULL);
+    else
+	conn = virConnectOpen(NULL);
     if (conn == NULL) {
         fprintf(stderr, "Second virConnectOpen() failed\n");
 	exit(1);
